// m: 
//  2  21.04.2014

function MainChain(canvas) {

    //  

    var context = canvas.getContext("2d");                  //  context  
    document.oncontextmenu=function(e){return false};       //   

    const Pi = 3.1415926;                   //  ""

    const m0 = 1;                           //  
    const T0 = 1;                           //   (   )
    const a0 = 1;                           //   ( )

    const k0 = 2 * Pi / T0;                 //  
    const C0 = m0 * k0 * k0;                //  

    // ***    ***

    const Ny = 5;						    //  ,      (     )
    const scale = canvas.height / Ny / a0;  //         
    const m = 1 * m0;                 	    // 
    const C = 1 * C0;                 	    // 
    var num = 24;                           //   

    // ***    ***

    const fps = 50;             	        // frames per second -     ( )
    const spf = 100;              		    // steps per frame   -      ( )
    const dt  = 0.045 * T0 / fps;      	    //   ( )

    //  

    var w = canvas.width / scale;		    //     
    var h = canvas.height / scale;		    //     

    //   

    var balls;                              //  
    var uGraph; var vGraph;                 //  
    var r, rScale, pDist, hC, steps;        //   
    var rScale13, rScaleShift;              // ___  ___
    MainChain.prototype.setNum = function(n){num = n};       //    
    MainChain.prototype.newSystem = function(conf){
        MainChain.prototype.actualConf = conf;
        r = a0*10/num;               	    //     
        rScale = r * scale;                 //     
        pDist = w/num;                      //    (  )
        hC = h/2 * scale;                   //  
        rScale13 = rScale*1.3;              // ___  ___
        rScaleShift = rScale/5.0;           // ___  ___
        steps = 0;                          //   

        balls = [];
        for (var i = 1; i < num + 1; i++) {
            var b = [];

            b.x0 = pDist*(0.5 + i-1);       //     
            b.fx = 0;                       // ,   
            b.vx = 0;                       // 
            b.uu = 0;                       //      

            conf(b, i);                     //     

            b.xx = b.x0 + b.uu;             //    
            b.x = b.xx*scale;               //   

            balls[i] = b;                   //     
        }

        //    
        balls[0] = balls[num];
        balls[num+1] = balls[1];

        //       (     )
        var sumVx = 0;
        for (var i0 = 1; i0 < num+1; i0++) sumVx += balls[i0].vx;
        var vxAverage = sumVx/num;
        for (var i1 = 1; i1 < num+1; i1++) balls[i1].vx -= vxAverage;

        //      u(n)
        var sumVx2 = 0;
        for (var i2 = 1; i2 < num+1; i2++) sumVx2 += balls[i2].vx*balls[i2].vx;
        const sigma2 = sumVx2 / num;
        const omega2 = C/m;
        const yLabel = 2*Math.sqrt(sigma2/omega2) *(num/8);     //   - 

        uGraph = new TM_graph(              //  
            "#uGraph",                      //  html- #uGraph
            "n", "u",                       //   
            null                            //      
            ,-yLabel, +yLabel, yLabel/4     // .   Y, .   Y,    Y
        );

        vGraph = new TM_graph(              //  
            "#vGraph",                      //  html- #vGraph
            "n", "v",                       //   
            null                            //      
            ,-1.2, 1.2, 1.2/4               // .   Y, .   Y,    Y
        );
    };

    //   
    var v0 = 1*a0/T0;                       // conf_random -   
    var sinNum = 2;                         // conf_sin -     
    var hillDiv = 1/4;                      // conf_hill -  () ,   ""

    // 
    MainChain.prototype.conf_random = function(b){b.vx = v0*(2*Math.random()-1);};
    MainChain.prototype.conf_sin = function(b, i){b.vx = Math.sin(2*Pi * i/num*sinNum);};
    MainChain.prototype.conf_one = function(b, i){if (i == Math.ceil(num/2)) b.vx = 0.5; else b.vx = 0;};
    MainChain.prototype.conf_stair2 = function(b, i){if (i%4 == 0 || (i-1)%4 == 0) b.vx = 1; else b.vx = 0;};
    MainChain.prototype.conf_stair3 = function(b, i){if (i%6 == 0 || (i-1)%6 == 0 || (i-2)%6 == 0) b.vx = 1; else b.vx = 0;};
    MainChain.prototype.conf_hill = function(b, i){
        var nd2 = num*hillDiv/2;
        if (i <= nd2*2) b.vx = (nd2*nd2 - (i-nd2)*(i-nd2))/(nd2*nd2);
        else b.vx = 0;
    };

    //   

    function control() {
        physics();
        draw();
    }

    //   

    function physics(){
        for (var s=1; s<=spf; s++) {// ,     
            for (var i=1; i<balls.length-1; i++){
                balls[i].fx = C*(balls[i+1].uu - 2*balls[i].uu + balls[i-1].uu);
                balls[i].vx += balls[i].fx / m * dt;
                balls[i].xx += balls[i].vx * dt;

                balls[i].x = balls[i].xx * scale;
            }
            //   
            for (var i2=1; i2 < balls.length-1; i2++) balls[i2].uu = balls[i2].xx - balls[i2].x0;
        }
        steps++;

        //       [x, y]
        var uData = []; var vData = [];
        for (var i0=1; i0<balls.length-1; i0++) uData[i0] = [i0, balls[i0].uu];
        for (var i1=1; i1<balls.length-1; i1++) vData[i1] = [i1, balls[i1].vx];

        if (steps % 4 == 0) uGraph.graph(uData);   //    
        if (steps % 4 == 0) vGraph.graph(vData);   //    
    }

    // 

    function draw(){
        context.clearRect(0, 0, w*scale, h*scale);          //  
        for (var i = 0; i < balls.length; i++){
            var b = balls[i];

            //       
            var gradient=context.createRadialGradient(b.x, hC, rScale13, b.x-rScaleShift, hC+rScaleShift,0);
            gradient.addColorStop(0,"#0000bb");
            gradient.addColorStop(1,"#44ddff");
            context.fillStyle=gradient;

            context.beginPath();
            context.arc(b.x, hC, r*scale,0,2*Math.PI);
            context.closePath();
            context.fill();
        }
    }

    //  

    MainChain.prototype.newSystem(MainChain.prototype.conf_hill);
    setInterval(control, 1000/fps);
}